import { runSelectQuery } from './SparqlApi';
import { prefixes } from './SparqlQueriesPrefixes';
import { endpoint, countQuery, facetResultSetQuery } from './SparqlQueriesGeneral';
import { manuscriptProperties, productionPlacesQuery, migrationsQuery } from './SparqlQueriesManuscripts';
import { workProperties } from './SparqlQueriesWorks';
import { personProperties } from './SparqlQueriesPeople';
import { organizationProperties } from './SparqlQueriesOrganizations';
import { placeProperties, placeQuery, allPlacesQuery } from './SparqlQueriesPlaces';
import { facetConfigs } from './FacetConfigs';
import { mapCount } from './Mappers';
import { makeObjectList } from './SparqlObjectMapper';
import { generateFilter } from './Filters';

export const getPaginatedResults = async ({
  resultClass,
  page,
  pagesize,
  uriFilters,
  spatialFilters,
  sortBy,
  sortDirection
}) => {
  const [ resultCount, paginatedData ] = await Promise.all([
    getResultCount(resultClass, uriFilters, spatialFilters),
    getPaginatedData({ resultClass, page, pagesize, uriFilters, spatialFilters, sortBy, sortDirection }),
  ]);
  return {
    resultCount: resultCount,
    pagesize: pagesize,
    page: page,
    results: paginatedData
  };
};

  // return Promise.all([
  //   getResultCount(resultClass, uriFilters, spatialFilters),
  //   getPaginatedData({ resultClass, page, pagesize, uriFilters, spatialFilters, sortBy, sortDirection }),
  // ])
  //   .then(data => {
  //     return {
  //       resultCount: data[0].count,
  //       pagesize: pagesize,
  //       page: page,
  //       results: data[1]
  //     };
  //   });


export const getAllResults = ({
  resultClass, // TODO: handle other classes than manuscripts
  facetClass,
  uriFilters,
  spatialFilters,
  variant
}) => {
  let q = '';
  let filterTarget = '';
  switch (variant) {
    case 'allPlaces':
      q = allPlacesQuery;
      filterTarget = 'id';
      break;
    case 'productionPlaces':
      q = productionPlacesQuery;
      filterTarget = 'manuscripts';
      break;
    case 'migrations':
      q = migrationsQuery;
      filterTarget = 'manuscript__id';
      break;
  }
  if (uriFilters == null && spatialFilters == null) {
    q = q.replace('<FILTER>', '# no filters');
  } else {
    q = q.replace('<FILTER>', generateFilter({
      facetClass: facetClass,
      uriFilters: uriFilters,
      spatialFilters: spatialFilters,
      filterTarget: filterTarget,
      facetID: null
    }));
  }
  // if (variant == 'productionPlaces') {
  //   console.log(prefixes + q)
  // }
  return runSelectQuery(prefixes + q, endpoint, makeObjectList);
};

const getResultCount = (resultClass, uriFilters, spatialFilters) => {
  let q = countQuery;
  q = q.replace('<RDF_TYPE>', facetConfigs[resultClass].rdfType);
  if (uriFilters == null && spatialFilters == null) {
    q = q.replace('<FILTER>', '# no filters');
  } else {
    q = q.replace('<FILTER>', generateFilter({
      resultClass: resultClass,
      facetClass: resultClass,
      uriFilters: uriFilters,
      spatialFilters: spatialFilters,
      filterTarget: 'id',
      facetID: null
    }));
  }
  return runSelectQuery(prefixes + q, endpoint, mapCount);
};

const getPaginatedData = ({
  resultClass,
  page,
  pagesize,
  uriFilters,
  spatialFilters,
  sortBy,
  sortDirection
}) => {
  let q = facetResultSetQuery;
  const facetConfig = facetConfigs[resultClass];
  if (uriFilters == null && spatialFilters == null) {
    q = q.replace('<FILTER>', '# no filters');
  } else {
    q = q.replace('<FILTER>', generateFilter({
      resultClass: resultClass,
      facetClass: resultClass,
      uriFilters: uriFilters,
      spatialFilters: spatialFilters,
      filterTarget: 'id',
      facetID: null}));
  }
  q = q.replace('<RDF_TYPE>', facetConfig.rdfType);
  q = q.replace('<ORDER_BY_PREDICATE>', facetConfig[sortBy].labelPath);
  q = q.replace('<SORT_DIRECTION>', sortDirection);
  q = q.replace('<PAGE>', `LIMIT ${pagesize} OFFSET ${page * pagesize}`);
  let resultSetProperties;
  switch (resultClass) {
    case 'manuscripts':
      resultSetProperties = manuscriptProperties;
      break;
    case 'works':
      resultSetProperties = workProperties;
      break;
    case 'places':
      resultSetProperties = placeProperties;
      break;
    case 'people':
      resultSetProperties = personProperties;
      break;
    case 'organizations':
      resultSetProperties = organizationProperties;
      break;
    default:
      resultSetProperties = '';
  }
  q = q.replace('<RESULT_SET_PROPERTIES>', resultSetProperties);
  // console.log(prefixes + q)
  return runSelectQuery(prefixes + q, endpoint, makeObjectList);
};

export const getByURI = ({
  resultClass,
  facetClass,
  variant,
  uriFilters,
  uri
}) => {
  let q;
  switch (resultClass) {
    case 'places':
      q = placeQuery;
      break;
  }
  if (variant === 'productionPlaces') {
    const manuscriptsProduced =
      `OPTIONAL {
          ${generateFilter(resultClass, facetClass, uriFilters, 'manuscript__id', null)}
          ?manuscript__id ^crm:P108_has_produced/crm:P7_took_place_at ?id .
          ?manuscript__id mmm-schema:data_provider_url ?manuscript__dataProviderUrl .
        }`;
    q = q.replace('<MANUSCRIPTS>', manuscriptsProduced);
  } else {
    q = q.replace('<MANUSCRIPTS>', '');
  }
  q = q.replace('<ID>', `<${uri}>`);
  // if (variant === 'productionPlaces') {
  //   console.log(prefixes + q)
  // }
  return runSelectQuery(prefixes + q, endpoint, makeObjectList);
};
